package com.fr.plugin.xx.zyjn.decode.encode.bi;

import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.webservice.v10.config.ConfigService;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.xx.zyjn.decode.LogUtils;
import com.fr.plugin.xx.zyjn.decode.util.GZIPUtils;
import com.fr.stable.StringUtils;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @Author xx
 * @Date 2022/3/3
 * @Description
 **/
public class EncodeFilter extends AbstractGlobalRequestFilterProvider {
    @Override
    public String filterName() {
        return "encode";
    }

    @Override
    public String[] urlPatterns() {
        String servletPathName = "decision";
        try {
            servletPathName = ConfigService.getInstance().getBasicParam().getServletPathName();
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return new String[]{
                "/" + servletPathName + "/v5/design/report/data/export",
                "/" + servletPathName + "/v5/design/report/data/global/export/pdf",
                "/" + servletPathName + "/v5/design/report/data/global/export/excel",

        };
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {
        try {
            String method = req.getMethod();
            if ("POST".equalsIgnoreCase(method)) {
                String uri = req.getRequestURI();
                String query = req.getQueryString();
                LogUtils.debug4plugin("BI export url {}",uri + "?" + query);
                ResponseWrapperImpl wrapper = new ResponseWrapperImpl(res);
                filterChain.doFilter(req, wrapper);
                FileUtil.handle(res.getOutputStream(), os -> {
                    byte[] bytes = wrapper.getDataStream();
                    if (StringUtils.contains(wrapper.getHeader("Content-Encoding"), "gzip")) {
                        FineLoggerFactory.getLogger().info("current url {} is gzip", getUrl(req));
                        bytes = GZIPUtils.decompress(bytes);
                    }
                    os.write(bytes);
                    os.flush();
                }, req,res);
            }
        } catch (Exception e) {
            LogUtils.error(e.getMessage(), e);
        }
    }

    private String getUrl(HttpServletRequest req) {
        String uri = req.getRequestURI();
        String query = req.getQueryString();
        return StringUtils.isNotBlank(query) ? uri + "?" + query : uri;
    }
}
