package com.fr.plugin.xx.zyjn.decode.encode.bi;

import com.fr.general.CommonIOUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.xx.zyjn.decode.LogUtils;
import com.fr.plugin.xx.zyjn.decode.encode.EncryptOutputStream;
import com.fr.plugin.xx.zyjn.decode.util.GZIPUtils;
import com.fr.stable.StringUtils;
import com.fr.third.org.apache.commons.io.IOUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.Arrays;
import java.util.UUID;

/**
 * @author xx
 * @date 2020/11/20
 */
public class FileUtil {

    /**
     * 文件加密之后再写入指定的输出流
     *
     * @param file
     * @param stream
     * @return void
     **/
    public static void handle(File file, OutputStream stream, HttpServletRequest req, HttpServletResponse res) {
        InputStream is = null;
        try {
            FileInputStream inputStream = new FileInputStream(file);
            byte[] encodeFile = encodeFile(CommonIOUtils.inputStream2Bytes(inputStream), req);
            is = new ByteArrayInputStream(encodeFile);
            byte[] bytes = CommonIOUtils.inputStream2Bytes(is);
            if (StringUtils.contains(res.getHeader("Content-Encoding"), "gzip")) {
                bytes = GZIPUtils.compress(bytes);
            }
            IOUtils.copyLarge(new ByteArrayInputStream(bytes), stream);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    FineLoggerFactory.getLogger().error(e.getMessage(), e);
                }
            }
            if (stream != null) {
                try {
                    stream.flush();
                } catch (IOException e) {
                    FineLoggerFactory.getLogger().error(e.getMessage(), e);
                }
            }
        }
    }

    private static byte[] encodeFile(byte[] file, HttpServletRequest req) {
        try {
            return EncryptOutputStream.encrypt(file,true);
        } catch (Exception e) {
            LogUtils.error(e.getMessage(), e);
        }
        return file;
    }

    /**
     * 加密导出文件后写入原始输出流中
     * 回调函数式处理
     *
     * @param outputStream 原始输出流
     * @param func         临时输出流处理
     * @return void
     **/
    public static void handle(OutputStream outputStream, ExportFunc func, HttpServletRequest req, HttpServletResponse res) {
        File tempFile = null;
        OutputStream os = null;
        //写入到临时文件
        try {
            tempFile = FileUtil.createTempFile();
            os = new FileOutputStream(tempFile);
            //直接写到临时文件里边
            func.export(os);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        } finally {
            if (os != null) {
                try {
                    os.close();
                } catch (IOException e) {
                    FineLoggerFactory.getLogger().error(e.getMessage(), e);
                }
            }
        }

        //处理临时文件，写到原始输出流
        try {
            handle(tempFile, outputStream, req,res);//临时加密后写到原始输出流中
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        } finally {
            if (tempFile != null) {
                deleteFile(tempFile);//最后删除临时文件
            }
        }
    }

    /**
     * 创建临时文件
     *
     * @return java.io.File
     **/
    public static File createTempFile() throws IOException {
        return File.createTempFile("tmp" + UUID.randomUUID().toString(), null);//创建一个临时文件
    }

    /**
     * 删除文件
     *
     * @param file
     * @return boolean
     * @author Zhanying
     **/
    public static boolean deleteFile(File file) {
        boolean flag = file.exists() && file.isFile() && file.delete();
        // 如果文件路径所对应的文件存在，并且是一个文件，则直接删除
        return flag;
    }

    private static byte[] ZIP_HEADER_1 = new byte[]{80, 75, 3, 4};

    private static byte[] ZIP_HEADER_2 = new byte[]{80, 75, 5, 6};

    public static boolean isArchiveFile(byte[] file) {
        if (file == null) {
            return false;
        }
        boolean isArchive = false;
        InputStream input = null;
        try {
            input = new ByteArrayInputStream(file);
            byte[] buffer = new byte[4];
            int length = input.read(buffer, 0, 4);
            if (length == 4) {
                isArchive = (Arrays.equals(ZIP_HEADER_1, buffer)) || (Arrays.equals(ZIP_HEADER_2, buffer));
            }
        } catch (IOException e) {
            LogUtils.error(e.getMessage(), e);
        } finally {
            if (input != null) {
                try {
                    input.close();
                } catch (IOException e) {
                }
            }
        }
        return isArchive;
    }


    public static File createTempFolder(String path) {
        File file = new File(System.getProperty("java.io.tmpdir")+File.separator+path);
        file.mkdir();
        return file;
    }
}
